@@ -36,7 +36,10 @@ except Exception:
     pass
 
 sys.path.insert(0, '.')
-from hooks import HOOKS  # noqa
+try:
+    from hooks import HOOKS  # noqa
+except ImportError:
+    pass
 
 
 EXTENSIONS = ['oslo.cache',
@@ -56,6 +59,11 @@ _TYPE_DESCRIPTIONS = {
     cfg.ListOpt: 'List',
     cfg.DictOpt: 'Dict',
     cfg.MultiStrOpt: 'Multi-valued',
+    cfg.IPOpt: 'IP address',
+    cfg.PortOpt: 'Port number',
+    cfg.HostnameOpt: 'Hostname',
+    cfg.URIOpt: 'URI',
+    cfg.HostAddressOpt: 'Host address',
     cfg._ConfigFileOpt: 'List of filenames',
     cfg._ConfigDirOpt: 'List of directory names',
 }
@@ -131,7 +139,7 @@ def import_modules(repo_location, package_name, verbose=0):
                     continue
                 except Exception as e:
                     print("Impossible to import module %s" % modname)
-                    raise e
+                    raise
 
                 _register_runtime_opts(module, abs_path, verbose)
                 _run_hook(modname)
@@ -187,7 +195,7 @@ def _register_runtime_opts(module, abs_path, verbose):
                               {'obj': opts_var, 'module': module})
                     break
 
-            if register:
+            if register and isinstance(obj, list):
                 for opt in obj:
                     if not isinstance(opt, cfg.Opt):
                         continue
@@ -207,7 +215,7 @@ def _sanitize_default(opt):
         return str(opt.sample_default)
 
     if ((type(opt).__name__ == "ListOpt") and (type(opt.default) == list)):
-        return ", ".join(opt.default)
+        return ", ".join(str(item) for item in opt.default)
 
     default = str(opt.default)
 
@@ -259,6 +267,7 @@ class OptionsCache(object):
     def __init__(self, overrides={}, verbose=0):
         self._verbose = verbose
         self._opts_by_name = {}
+        self._opts_by_group = {}
         self._opt_names = []
         self._overrides = overrides
 
@@ -278,7 +287,7 @@ class OptionsCache(object):
     def _add_opt(self, optname, group, opt):
         if optname in self._opts_by_name:
             if self._verbose >= 2:
-                print ("Duplicate option name %s" % optname)
+                print("Duplicate option name %s" % optname)
             return
 
         opt.default = _sanitize_default(opt)
@@ -289,6 +298,11 @@ class OptionsCache(object):
             self._opts_by_name[optname] = (group, opt)
             self._opt_names.append(optname)
 
+            if group not in self._opts_by_group:
+                self._opts_by_group[group] = []
+
+            self._opts_by_group[group].append(opt)
+
         if optname in self._overrides:
             for new_group in self._overrides[optname]:
                 if new_group == 'DEFAULT':
@@ -336,9 +350,15 @@ class OptionsCache(object):
         for ext in needed_exts:
             self.load_extension_options(ext)
 
+    def get_group_names(self):
+        return self._opts_by_group.keys()
+
     def get_option_names(self):
         return self._opt_names
 
+    def get_group(self, name):
+        return self._opts_by_group[name]
+
     def get_option(self, name):
         return self._opts_by_name[name]
 
@@ -377,10 +397,8 @@ class OptionsCache(object):
             return cmp(x, y)
 
 
-def pass_through(line):
-    """Whether to ignore the line."""
-    return (not line.strip() or
-            line.startswith('#'))
+def _use_categories(package_name):
+    return not os.path.isfile(package_name + '.disable')
 
 
 def _get_options_by_cat(package_name):
@@ -388,7 +406,7 @@ def _get_options_by_cat(package_name):
 
     with open(package_name + '.flagmappings') as f:
         for line in f:
-            if pass_through(line):
+            if not line.strip() or line.startswith('#'):
                 continue
             opt, categories = line.split(' ', 1)
             for category in categories.split():
@@ -404,7 +422,7 @@ def _get_category_names(package_name):
         try:
             with open(headers_file) as f:
                 for line in f:
-                    if pass_through(line):
+                    if not line.strip() or line.startswith('#'):
                         continue
                     cat, nice_name = line.split(' ', 1)
                     category_names[cat] = nice_name.strip()
@@ -414,30 +432,86 @@ def _get_category_names(package_name):
     return category_names
 
 
-def write_files(package_name, options, target, output_format):
+def _format_opt(option):
+
+    def _remove_prefix(text, prefix):
+        if text.startswith(prefix):
+            return text[len(prefix):].lstrip(':')
+        return text
+
+    def _reflow_text(text):
+        text = re.sub(r'\n+\s*\* ', '$sentinal$* ', text)
+        text = text.replace('\n\n', '$sentinal$')
+        text = text.replace('\n', ' ')
+        text = ' '.join(text.split())
+        return text.split('$sentinal$')
+
+    def _strip_indentation(text):
+        return ' '.join([x.strip() for x in text.split('\n')]).strip()
+
+    help_text = option.help or "No help text available for this option."
+    help_text = _remove_prefix(help_text.strip(), 'DEPRECATED')
+    help_text = _reflow_text(help_text)
+
+    deprecated_text = (option.deprecated_reason or
+                       'No deprecation reason provided for this option.')
+    deprecated_text = _strip_indentation(deprecated_text)
+
+    opt_type = _TYPE_DESCRIPTIONS.get(type(option), 'Unknown')
+
+    flags = []
+
+    if (option.deprecated_for_removal or
+            (option.help and option.help.startswith('DEPRECATED'))):
+        flags.append(('Deprecated', deprecated_text))
+    if option.mutable:
+        flags.append(('Mutable', 'This option can be changed without'
+                      ' restarting.'))
+
+    return {
+        'name': option.dest,
+        'type': opt_type,
+        'default': _sanitize_default(option),
+        'help': help_text,
+        'flags': flags
+    }
+
+
+def write_files(package_name, options, target):
     """Write tables.
 
-    Prints a table for every group of options.
+    Some projects make use of flagmapping files, while others make use
+    of oslo.config's OptGroup to do the same in code. The function will
+    handle both, printing a list of options by either category or group.
     """
-    if not target:
-        if output_format == 'rst':
-            target = '../../doc/config-reference/source/tables'
-        else:
-            target = '../../doc/common/tables/'
-    options_by_cat = _get_options_by_cat(package_name)
-    category_names = _get_category_names(package_name)
-
+    target = target or '../../doc/config-reference/source/tables'
     if not os.path.isdir(target):
         os.makedirs(target)
 
+    if _use_categories(package_name):
+        _write_files_by_category(package_name, options, target)
+    else:
+        _write_files_by_group(package_name, options, target)
+
+
+def _write_files_by_category(package_name, options, target):
+    options_by_cat = _get_options_by_cat(package_name)
+    category_names = _get_category_names(package_name)
+
     for cat in options_by_cat.keys():
         env = {
             'pkg': package_name,
             'cat': cat,
+            'label': '-'.join([package_name, cat]),
             'groups': [],
             'items': [],
         }
 
+        # Skip the options that is explicitly marked as disabled,
+        # which is used for common configuration options.
+        if cat == 'disable':
+            continue
+
         if cat in category_names:
             env['nice_cat'] = category_names[cat]
         else:
@@ -457,37 +531,45 @@ def write_files(package_name, options, target, output_format):
                         env['items'].append(items)
                 items = []
 
-            if not option.help:
-                option.help = "No help text available for this option."
-            helptext = option.help.strip().replace('\n', ' ')
-            helptext = ' '.join(helptext.split())
-            if option.deprecated_for_removal:
-                if not option.help.strip().startswith('DEPRECATED'):
-                    helptext = 'DEPRECATED: ' + helptext
-                if option.deprecated_reason:
-                    helptext += ' ' + option.deprecated_reason
-
-            opt_type = _TYPE_DESCRIPTIONS.get(type(option), 'Unknown')
-            item = (option.dest,
-                    _sanitize_default(option),
-                    "(%s) %s" % (opt_type, helptext))
-            items.append(item)
+            items.append(_format_opt(option))
 
         env['items'].append(items)
-        env['table_label'] = package_name + '-' + cat
 
-        ext = 'rst' if output_format == 'rst' else 'xml'
-        file_path = ("%(target)s/%(package_name)s-%(cat)s.%(ext)s" %
+        file_path = ("%(target)s/%(package_name)s-%(cat)s.rst" %
                      {'target': target, 'package_name': package_name,
-                      'cat': cat, 'ext': ext})
+                      'cat': cat})
         tmpl_file = os.path.join(os.path.dirname(__file__),
-                                 'templates/autohelp.%s.j2' % output_format)
-        with open(tmpl_file) as fd:
-            template = jinja2.Template(fd.read(), trim_blocks=True)
-            output = template.render(filename=file_path, **env)
+                                 'templates/autohelp-category.rst.j2')
+        _write_template(tmpl_file, file_path, env)
 
-        with open(file_path, 'w') as fd:
-            fd.write(output)
+
+def _write_files_by_group(package_name, options, target):
+    for group in options.get_group_names():
+        env = {
+            'pkg': package_name,
+            'group': group,
+            'label': '-'.join([package_name, group]),
+            'items': [],
+        }
+
+        for option in options.get_group(group):
+            env['items'].append(_format_opt(option))
+
+        file_path = ("%(target)s/%(package_name)s-%(group)s.rst" %
+                     {'target': target, 'package_name': package_name,
+                      'group': group})
+        tmpl_file = os.path.join(os.path.dirname(__file__),
+                                 'templates/autohelp-group.rst.j2')
+        _write_template(tmpl_file, file_path, env)
+
+
+def _write_template(template_path, output_path, env):
+    with open(template_path) as fd:
+        template = jinja2.Template(fd.read(), trim_blocks=True)
+        output = template.render(filename=output_path, **env)
+
+    with open(output_path, 'w') as fd:
+        fd.write(output)
 
 
 def update_flagmappings(package_name, options, verbose=0):
@@ -497,6 +579,10 @@ def update_flagmappings(package_name, options, verbose=0):
     This will create a new file $package_name.flagmappings.new with
     category information merged from the existing $package_name.flagmappings.
     """
+    if not _use_categories:
+        print("This project does not use flagmappings. Nothing to update.")
+        return
+
     original_flags = {}
     try:
         with open(package_name + '.flagmappings') as f:
@@ -543,8 +629,8 @@ def main():
         description='Manage flag files, to aid in updating documentation.',
         usage='%(prog)s <cmd> <package> [options]')
     parser.add_argument('subcommand',
-                        help='Action (update, docbook, rst, dump).',
-                        choices=['update', 'docbook', 'rst', 'dump'])
+                        help='Action (update, rst, dump).',
+                        choices=['update', 'rst', 'dump'])
     parser.add_argument('package',
                         help='Name of the top-level package.')
     parser.add_argument('-v', '--verbose',
@@ -564,7 +650,7 @@ def main():
                         dest='target',
                         help='Directory or file in which data will be saved.\n'
                              'Defaults to ../../doc/common/tables/ '
-                             'for "docbook".\n'
+                             'for "rst".\n'
                              'Defaults to stdout for "dump"',
                         required=False,
                         type=str,)
@@ -599,8 +685,8 @@ def main():
     if args.subcommand == 'update':
         update_flagmappings(args.package, options, verbose=args.verbose)
 
-    elif args.subcommand in ('docbook', 'rst'):
-        write_files(args.package, options, args.target, args.subcommand)
+    elif args.subcommand == 'rst':
+        write_files(args.package, options, args.target)
 
     elif args.subcommand == 'dump':
         options.dump()
