@@ -26,9 +26,8 @@ import jinja2
 
 
 PROJECTS = ['aodh', 'ceilometer', 'cinder', 'glance', 'heat', 'ironic',
-            'keystone', 'manila', 'neutron', 'nova', 'sahara', 'swift',
-            'trove']
-MASTER_RELEASE = 'Mitaka'
+            'keystone', 'manila', 'neutron', 'nova', 'sahara', 'trove']
+MASTER_RELEASE = 'Ocata'
 CODENAME_TITLE = {'aodh': 'Alarming',
                   'ceilometer': 'Telemetry',
                   'cinder': 'Block Storage',
@@ -37,11 +36,13 @@ CODENAME_TITLE = {'aodh': 'Alarming',
                   'ironic': 'Bare Metal service',
                   'keystone': 'Identity service',
                   'manila': 'Shared File Systems service',
+                  'murano': 'Application Catalog service',
                   'neutron': 'Networking',
                   'nova': 'Compute',
                   'sahara': 'Data Processing service',
-                  'swift': 'Object Storage service',
-                  'trove': 'Database service'}
+                  'senlin': 'Clustering service',
+                  'trove': 'Database service',
+                  'zaqar': 'Message service'}
 
 
 def setup_venv(projects, branch, novenvupdate):
@@ -162,14 +163,7 @@ def format_option_name(name):
         # name without a section ('log_dir')
         return "[DEFAULT] %s" % name
 
-    try:
-        # If we're dealing with swift, we'll have a filename to extract
-        # ('proxy-server|filter:tempurl/use')
-        filename, section = section.split('|')
-        return "%s.conf: [%s] %s" % (filename, section, name)
-    except ValueError:
-        # section but no filename ('database/connection')
-        return "[%s] %s" % (section, name)
+    return "[%s] %s" % (section, name)
 
 
 def release_from_branch(branch):
@@ -257,13 +251,6 @@ def main():
                         required=False,
                         default='.',
                         type=str,)
-    parser.add_argument('-f', '--format',
-                        dest='format',
-                        help='Output format (docbook or rst).',
-                        required=False,
-                        default='rst',
-                        type=str,
-                        choices=('docbook', 'rst'),)
     parser.add_argument('-n', '--no-venv-update',
                         dest='novenvupdate',
                         help='Don\'t update the virtual envs.',
@@ -281,10 +268,9 @@ def main():
 
         release = args.new_branch.replace('stable/', '')
         env = get_env(project, release, old_list, new_list)
-        ext = 'rst' if args.format == 'rst' else 'xml'
-        filename = ("%(project)s-conf-changes.%(ext)s" %
-                    {'project': project, 'ext': ext})
-        tmpl_file = 'templates/changes.%s.j2' % args.format
+        filename = ("%(project)s-conf-changes.rst" %
+                    {'project': project})
+        tmpl_file = 'templates/changes.rst.j2'
         if not os.path.exists(args.target):
             os.makedirs(args.target)
         dest = os.path.join(args.target, filename)
